<?php
/**
 * @file
 * @package sd
 * @version $Id$
**/

if(!defined('XOOPS_ROOT_PATH'))
{
    exit;
}

require_once SD_TRUST_PATH . '/class/AbstractAction.class.php';

define('SD_FRAME_PERFORM_SUCCESS', 1);
define('SD_FRAME_PERFORM_FAIL', 2);
define('SD_FRAME_INIT_SUCCESS', 3);

define('SD_FRAME_VIEW_NONE','none');
define('SD_FRAME_VIEW_SUCCESS','success');
define('SD_FRAME_VIEW_ERROR','error');
define('SD_FRAME_VIEW_INDEX','index');
define('SD_FRAME_VIEW_INPUT','input');
define('SD_FRAME_VIEW_PREVIEW','preview');
define('SD_FRAME_VIEW_CANCEL','cancel');

/**
 * Sd_Module
**/
class Sd_Module extends Legacy_ModuleAdapter
{
    /**
     * @var  string
     * 
     * @public
    **/
    var $mActionName = null;

    /**
     * @var  Sd_AbstractAction
     * 
     * @public
    **/
    var $mAction = null;

    /**
     * @var  bool
     * 
     * @public
    **/
    var $mAdminFlag = false;

    /**
     * @var  Sd_AssetManager
     * 
     * @public
    **/
    var $mAssetManager = null;

    /**
     * @var  Enum[]
     * 
     * @protected
    **/
    var $_mAllowViewNames = array(
        SD_FRAME_VIEW_NONE,
        SD_FRAME_VIEW_SUCCESS,
        SD_FRAME_VIEW_ERROR,
        SD_FRAME_VIEW_INDEX,
        SD_FRAME_VIEW_INPUT,
        SD_FRAME_VIEW_PREVIEW,
        SD_FRAME_VIEW_CANCEL
    );

    /**
     * startup
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @public
    **/
    function startup()
    {
        parent::startup();
    
        XCube_DelegateUtils::call(
            'Module.sd.Event.GetAssetManager',
            new XCube_Ref($this->mAssetManager),
            $this->mXoopsModule->get('dirname')
        );
    
        $root =& XCube_Root::getSingleton();
        $root->mController->mExecute->add(array(&$this, 'execute'));
    
        //
        // TODO/Insert your initialization code.
        //
    }

    /**
     * setAdminMode
     * 
     * @param   bool  $flag
     * 
     * @return  void
     * 
     * @public
    **/
    function setAdminMode($flag)
    {
        $this->mAdminFlag = $flag;
    }

    /**
     * _getDefaultActionName
     * 
     * @param   void
     * 
     * @return  string
     * 
     * @private
    **/
    function _getDefaultActionName()
    {
        // TODO insert your default action name
        return 'index';
    }

    /**
     * setActionName
     * 
     * @param   string  $name
     * 
     * @return  void
     * 
     * @public
    **/
    function setActionName($name)
    {
        $this->mActionName = $name;
    }

    /**
     * getRenderSystemName
     * 
     * @param   void
     * 
     * @return  string
     * 
     * @public
    **/
    function getRenderSystemName()
    {
        if(!$this->mAdminFlag)
        {
            return parent::getRenderSystemName();
        }
    
        // TODO will be use site config
        if(!defined('SD_ADMIN_RENDER_REGISTED'))
        {
            define('SD_ADMIN_RENDER_REGISTED',true);
            $root =& XCube_Root::getSingleton();
            $root->overrideSiteConfig(
                array(
                    'RenderSystems' => array(
                        'Sd_AdminRenderSystem' => 'Sd_AdminRenderSystem'
                    ),
                    'Sd_AdminRenderSystem' => array(
                        'root' => SD_TRUST_PATH,
                        'path' => '/admin/class/SdAdminRenderSystem.class.php',
                        'class' => 'Sd_AdminRenderSystem'
                    )
                )
            );
        }
    
        return 'Sd_AdminRenderSystem';
    }

    /**
     * getAdminMenu
     * 
     * @param   void
     * 
     * @return  {string 'title',string 'link',string 'keywords',bool 'show',bool 'absolute'}[]
     * 
     * @public
    **/
    function getAdminMenu()
    {
        if(is_array($this->mAdminMenu))
        {
            return $this->mAdminMenu;
        }
    
        $root =& XCube_Root::getSingleton();
    
        // load admin menu
        $adminMenu = $this->mXoopsModule->getInfo('adminmenu');
        if(!is_array($adminMenu))
        {
            $adminMenu = array();
        }
    
        // add preference menu
        if(is_array($this->mXoopsModule->getInfo('config')) && count($this->mXoopsModule->getInfo('config')) > 0)
        {
            $adminMenu[] = array(
                'title'    => _PREFERENCES,
                'link'     => $root->mController->getPreferenceEditUrl($this->mXoopsModule),
                'absolute' => true
            );
        }
    
        // add help menu
        if($this->mXoopsModule->hasHelp())
        {
            $adminMenu[] = array(
                'title'    => _HELP,
                'link'     => $root->mController->getHelpViewUrl($this->mXoopsModule),
                'absolute' => true
            );
        }
    
        $this->mAdminMenu = array();
        foreach($adminMenu as $menu)
        {
            if(!(isset($menu['absolute']) && $menu['absolute']))
            {
                $menu['link'] = XOOPS_MODULE_URL . '/' . $this->mXoopsModule->get('dirname') . '/' . $menu['link'];
            }
            $this->mAdminMenu[] = $menu;
        }
    
        return $this->mAdminMenu;
    }

    /**
     * execute
     * 
     * @param   XCube_Controller  &$controller
     * 
     * @return  void
     * 
     * @private
    **/
    function execute(&$controller)
    {
        if($this->_createAction() === false)
        {
            $this->doActionNotFoundError();
            die();
        }
    
        if($this->mAction->prepare() === false)
        {
            $this->doPreparationError();
            die();
        }
    
        if($this->mAction->hasPermission() === false)
        {
            $this->doPermissionError();
            die();
        }
    
        $viewStatus = (Sd_Utils::getEnv('REQUEST_METHOD') == 'POST') ?
            $this->mAction->execute() :
            $this->mAction->getDefaultView();
    
        if(in_array($viewStatus,$this->_mAllowViewNames))
        {
            $methodName = 'executeView' . ucfirst($viewStatus);
            if(is_callable(array($this->mAction,$methodName)))
            {
                $this->mAction->$methodName($this->getRenderTarget());
            }
        }
    }

    /**
     * _createAction
     * 
     * @param   void
     * 
     * @return  bool
     * 
     * @private
    **/
    function _createAction()
    {
        $root =& XCube_Root::getSingleton();
    
        if($this->mActionName == null)
        {
            $this->mActionName = $root->mContext->mRequest->getRequest('action');
            if($this->mActionName == null)
            {
                $this->mActionName = $this->_getDefaultActionName();
            }
        }
    
        if(!preg_match('/^\w+$/',$this->mActionName))
        {
            return false;
        }
    
        $fileName = ($this->mAdminFlag ? '/admin' : '')
            . '/actions/' . ucfirst($this->mActionName) . 'Action.class.php';
        switch(true)
        {
            case file_exists(
                $path = XOOPS_MODULE_PATH . '/' . $this->mXoopsModule->get('dirname') . $fileName
            ):
                break;
            case file_exists(
                $path = SD_TRUST_PATH . '/' . $fileName
            ):
                break;
            default:
                return false;
        }
    
        require_once $path;
    
        $className = 'Sd_' . ($this->mAdminFlag ? 'Admin_' : '')
            . ucfirst($this->mActionName) . 'Action';
        if(class_exists($className))
        {
            $this->mAction =& new $className();
        }
        if(!is_object($this->mAction) && !is_a($this->mAction,'Sd_AbstractAction'))
        {
            return false;
        }
    
        return true;
    }

    /**
     * doActionNotFoundError
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function doActionNotFoundError()
    {
        XCube_DelegateUtils::call(
            'Module.' . $this->mXoopsModule->get('dirname') . '.Event.Exception.ActionNotFound'
        );
        $root =& XCube_Root::getSingleton();
        $root->mController->executeForward(XOOPS_URL);
    }

    /**
     * doPreparationError
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function doPreparationError()
    {
        XCube_DelegateUtils::call(
            'Module.' . $this->mXoopsModule->get('dirname') . '.Event.Exception.Preparation'
        );
        $root =& XCube_Root::getSingleton();
        $root->mController->executeForward(XOOPS_URL);
    }

    /**
     * doPermissionError
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function doPermissionError()
    {
        XCube_DelegateUtils::call(
            'Module.' . $this->mXoopsModule->get('dirname') . '.Event.Exception.Permission'
        );
        $root =& XCube_Root::getSingleton();
        $root->mController->executeForward(XOOPS_URL);
    }
}

?>
