<?php
/**
 * @file
 * @package sd
 * @version $Id$
**/

if(!defined('XOOPS_ROOT_PATH'))
{
    exit();
}

/**
 * @class   Sd_ChardataUpdater
**/
class Sd_ChardataUpdater extends Sd_AbstractJsUpdater
{
    /**
     * @brief   Enum[]
    **/
    protected $_mHandlerList = array(
        SD_OBJECT_TYPE_CHAR,
        SD_OBJECT_TYPE_ACTIVE_ABILITY,
        SD_OBJECT_TYPE_IMAGE,
        SD_OBJECT_TYPE_ICON,
        SD_OBJECT_TYPE_ELEMENT,
        SD_OBJECT_TYPE_COSTUME,
        SD_OBJECT_TYPE_LEARNED_COSTUME,
        SD_OBJECT_TYPE_ABILITY,
        SD_OBJECT_TYPE_LEARNED_ABILITY,
        SD_OBJECT_TYPE_SKILL,
        SD_OBJECT_TYPE_LEARNED_SKILL,
        SD_OBJECT_TYPE_ENUM
    );
    
    /**
     * @brief   Sd_CharObject
    **/
    protected $_mChar = null;
    
    /**
     * @brief   string{}
    **/
    protected $_mAbilities = array();
    
    /**
     * @brief   string
    **/
    protected $_mActiveCostume = null;
    
    /**
     * @brief   Sd_ImageObject
    **/
    protected $_mImage = null;
    
    /**
     * @brief   Sd_ElementObject
    **/
    protected $_mElement = null;
    
    /**
     * _save
     * 
     * @param   void
     * 
     * @return  void
    **/
    protected function _save()
    {
        $this->_mChar->set('active_costume',$this->_mHandlers[SD_OBJECT_TYPE_COSTUME]->convertToData($this->_mActiveCostume));
        $this->_insert(SD_OBJECT_TYPE_CHAR,$this->_mChar);
        
        if(count($this->_mAbilities) > 0)
        {
            foreach($this->_mAbilities as $slot => $name)
            {
                $obj = $this->_mHandlers[SD_OBJECT_TYPE_ABILITY]->getByName($this->_convertString($name));
                $ability = $this->_mHandlers[SD_OBJECT_TYPE_ACTIVE_ABILITY]->create();
                $ability->setVars(
                    array(
                        'eno' => $this->_mActiveId,
                        'slot' => $slot,
                        'ability_id' => $obj->get('ability_id')
                    )
                );
                $this->_insert(SD_OBJECT_TYPE_ACTIVE_ABILITY,$ability);
            }
        }
        
        if($this->_mImage instanceof $this->_mHandlers[SD_OBJECT_TYPE_IMAGE]->mClass)
        {
            $this->_insert(SD_OBJECT_TYPE_IMAGE,$this->_mImage);
        }
        
        $this->_insert(SD_OBJECT_TYPE_ELEMENT,$this->_mElement);
    }
    
    /**
     * Parse_Show_Status
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Show_Status(/*** string ***/ $line)
    {
        $enumHandler =& $this->_mHandlers[SD_OBJECT_TYPE_ENUM];
        $arr = $this->_parseByRegex("/\((\d+),'(.+)','(.+)','([\d,]+)','([\d,]+)','(.+)(.+)(.+)','(.+)Lv.(\d+)','(.+)','(.+)','̡(\S+)( (\d+))?( (\d+)cm)?( (\d+)kg)? '\)$/",$line);
        
        $this->_mChar = $this->_mHandlers[SD_OBJECT_TYPE_CHAR]->create();
        
        $status = explode(',',$arr[4]);
        $this->_mChar->setVars(
            array(
                'eno'            => $this->_mActiveId,
                'full_name'      => $arr[2],
                'nick_name'      => $arr[3],
                'sex'            => $enumHandler->convertToData(SD_ENUM_TYPE_SEX,$arr[13]),
                'age'            => isset($arr[15]) ? $arr[15] : 0,
                'height'         => isset($arr[17]) ? $arr[17] : 0,
                'weight'         => isset($arr[19]) ? $arr[19] : 0,
                'kind'           => $enumHandler->convertToData(SD_ENUM_TYPE_KIND,$arr[8]),
                'element_base'   => $enumHandler->convertToData(SD_ENUM_TYPE_ELEMENT_BASE,$arr[6]),
                'element_ext'    => $enumHandler->convertToData(SD_ENUM_TYPE_ELEMENT_EXT,$arr[7]),
                'status_lv'      => $status[0],
                'status_life'    => $status[1],
                'status_soul'    => $status[2],
                'status_pow'     => $status[3],
                'status_str'     => $status[4],
                'status_dex'     => $status[5],
                'status_speed'   => $status[6],
                'status_mag'     => $status[7],
                'status_reg'     => $status[8],
                'status_con'     => $status[9],
                'status_exp'     => $status[10],
                'status_money'   => $status[11],
                'status_sp'      => $status[12]
            )
        );
        $this->_mActiveCostume = $arr[9];
        
        $abilities = explode(',',$arr[11]);
        for($i = 1;$i <= 5;$i++)
        {
            if($abilities[$i] != null)
            {
                $name = preg_replace('/\(\d+\)$/','',$abilities[$i]);
                /// @todo?
                if(!in_array($name,$this->_mAbilities))
                {
                    $this->_mAbilities[$i] = $name;
                }
            }
        }
    }
    
    /**
     * Parse_Show_Image
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Show_Image(/*** string ***/ $line)
    {
        $handler =& $this->_mHandlers[SD_OBJECT_TYPE_IMAGE];
        $arr = $this->_parseByRegex("/\((\d+),(\d+),'(.*)','(.*)'\);$/",$line);
        
        $oldImage = $handler->getByEno($this->_mActiveId);
        
        if($arr[3] != $oldImage->get('image') || $arr[4] != $oldImage->get('image_link'))
        {
            $newImage = $handler->create();
            
            $newImage->setVars(
                array(
                    'eno'          => $this->_mActiveId,
                    'image'        => $arr[3],
                    'image_height' => $arr[2],
                    'image_width'  => $arr[1],
                    'image_link'   => $arr[4],
                    'update_time'  => SD_UPDATE_TIME
                )
            );
            
            $this->_mImage =& $newImage;
        }
    }
    
    /**
     * Parse_Show_Element
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Show_Element(/*** string ***/ $line)
    {
        $arr = $this->_parseByExplode($line,24,13,-1);
        
        $this->_mElement = $this->_mHandlers[SD_OBJECT_TYPE_ELEMENT]->create();
        
        $arr[] = $this->_mActiveId;
        $this->_mElement->setVars(
            array_combine(
                array(
                    'ele_at_aqua',
                    'ele_at_flame',
                    'ele_at_wood',
                    'ele_at_wind',
                    'ele_at_iron',
                    'ele_at_life',
                    'ele_at_earth',
                    'ele_at_thunder',
                    'ele_at_light',
                    'ele_at_star',
                    'ele_at_dark',
                    'ele_at_flower',
                    'ele_df_aqua',
                    'ele_df_flame',
                    'ele_df_wood',
                    'ele_df_wind',
                    'ele_df_iron',
                    'ele_df_life',
                    'ele_df_earth',
                    'ele_df_thunder',
                    'ele_df_light',
                    'ele_df_star',
                    'ele_df_dark',
                    'ele_df_flower',
                    'eno'
                ),
                $arr
            )
        );
    }
    
    /**
     * Parse_Show_BattleStatus
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Show_BattleStatus(/*** string ***/ $line)
    {
        $arr = $this->_parseByExplode($line,22,18,-1);
        array_shift($arr);
        
        $this->_mChar->setVars(
            array_combine(
                array(
                    'hp_base',
                    'at_base',
                    'df_base',
                    'mat_base',
                    'mdf_base',
                    'act_base',
                    'move_base',
                    'hp_modify',
                    'at_modify',
                    'df_modify',
                    'mat_modify',
                    'mdf_modify',
                    'act_modify',
                    'move_modify',
                    'hp_equip',
                    'at_equip',
                    'df_equip',
                    'mat_equip',
                    'mdf_equip',
                    'act_equip',
                    'move_equip',
                ),
                $arr
            )
        );
    }
    
    /**
     * Parse_Costume_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Costume_Data(/*** string ***/ $line)
    {
        $costumeHandler =& $this->_mHandlers[SD_OBJECT_TYPE_COSTUME];
        $arr = $this->_parseByRegex("/\((\d+),'(.+)',(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)\)$/",$line);
        
        $costume = $costumeHandler->get($arr[1]);
        if(!$costume instanceof $costumeHandler->mClass)
        {
            $costume = $costumeHandler->create();
            $costume->setVars(
                array(
                    'costume_id'   => $arr[1],
                    'costume_name' => $arr[2],
                    'grow_life'    => $arr[5],
                    'grow_soul'    => $arr[6],
                    'grow_pow'     => $arr[7],
                    'grow_str'     => $arr[8],
                    'grow_dex'     => $arr[9],
                    'grow_speed'   => $arr[10],
                    'grow_mag'     => $arr[11],
                    'grow_reg'     => $arr[12],
                    'grow_con'     => $arr[13],
                    'modify_hp'    => $arr[14],
                    'modify_tp'    => $arr[15],
                    'modify_at'    => $arr[16],
                    'modify_df'    => $arr[17],
                    'modify_mat'   => $arr[18],
                    'modify_mdf'   => $arr[19],
                    'modify_act'   => $arr[20],
                    'move_cost'    => $arr[21],
                    'regist_time'  => SD_UPDATE_TIME
                )
            );
            $this->_insert(SD_OBJECT_TYPE_COSTUME,$costume);
        }
        
        $costume = $this->_mHandlers[SD_OBJECT_TYPE_LEARNED_COSTUME]->getByUnique($this->_mActiveId,$arr[1]);
        if($costume->get('costume_lv') != $arr[3] || $costume->get('costume_point') != $arr[4])
        {
            $costume->setVars(
                array(
                    'eno'           => $this->_mActiveId,
                    'costume_id'    => $arr[1],
                    'costume_lv'    => $arr[3],
                    'costume_point' => $arr[4]
                )
            );
            $this->_insert(SD_OBJECT_TYPE_LEARNED_COSTUME,$costume);
        }
    }
    
    /**
     * Parse_SAbility_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_SAbility_Data(/*** string ***/ $line)
    {
        $abilityHandler =& $this->_mHandlers[SD_OBJECT_TYPE_ABILITY];
        $learnedAbilityHandler =& $this->_mHandlers[SD_OBJECT_TYPE_LEARNED_ABILITY];
        $arr = $this->_parseByRegex("/\((\d+),'(.+)','(.+)',(\d+)\)$/",$line);
        
        $ability = $abilityHandler->get($arr[1]);
        if(!$ability instanceof $abilityHandler->mClass)
        {
            $ability = $abilityHandler->create();
            $ability->setVars(
                array(
                    'ability_id'   => $arr[1],
                    'ability_name' => $this->_convertString($arr[2]),
                    'description'  => $arr[3],
                    'ability_cost' => $arr[4],
                    'regist_time'  => SD_UPDATE_TIME
                )
            );
            $this->_insert(SD_OBJECT_TYPE_ABILITY,$ability);
        }
        
        $ability = $learnedAbilityHandler->getByUnique($this->_mActiveId,$arr[1]);
        if(!$ability instanceof $learnedAbilityHandler->mClass)
        {
            $ability = $learnedAbilityHandler->create();
            $ability->setVars(
                array(
                    'eno'        => $this->_mActiveId,
                    'ability_id' => $arr[1]
                )
            );
            $this->_insert(SD_OBJECT_TYPE_LEARNED_ABILITY,$ability);
        }
    }
    
    /**
     * Parse_WSkill_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_WSkill_Data(/*** string ***/ $line)
    {
        $skillHandler =& $this->_mHandlers[SD_OBJECT_TYPE_SKILL];
        $learnedSkillHandler =& $this->_mHandlers[SD_OBJECT_TYPE_LEARNED_SKILL];
        $enumHandler =& $this->_mHandlers[SD_OBJECT_TYPE_ENUM];
//        $arr = $this->_parseByRegex("/\((\d+),'(.+)','(.*)',(.+),'(.+)','[\d,]+',(\-?\d+),'(.+)',(\d+)\)$/",$line);
        $arr = $this->_parseByRegex("/\((\d+),'(.+)','(.*)',(.+),'(.+)','[\d,]+',(.+),'(.+)',(\d+)\)$/",$line);
        
        $skill = $skillHandler->get($arr[1]);
        if(!$skill instanceof $skillHandler->mClass)
        {
            $skill = $skillHandler->create();
            $skill->setVars(
                array(
                    'skill_id'    => $arr[1],
                    'skill_name'  => $this->_convertString($arr[2]),
                    'description' => $arr[3],
                    'range'       => trim($arr[4],"'") * 10,
                    'target'      => $enumHandler->convertToData(SD_ENUM_TYPE_TARGET,$arr[5]),
                    'element'     => trim($arr[6],"'"),
                    'kind'        => $enumHandler->convertToData(SD_ENUM_TYPE_TYPE,$arr[7]),
                    'tp'          => $arr[8],
                    'regist_time' => SD_UPDATE_TIME
                )
            );
            $this->_insert(SD_OBJECT_TYPE_SKILL,$skill);
        }
        
        $skill = $learnedSkillHandler->getByUnique($this->_mActiveId,$arr[1]);
        if(!$skill instanceof $learnedSkillHandler->mClass)
        {
            $skill = $learnedSkillHandler->create();
            $skill->setVars(
                array(
                    'eno'      => $this->_mActiveId,
                    'skill_id' => $arr[1]
                )
            );
            $this->_insert(SD_OBJECT_TYPE_LEARNED_SKILL,$skill);
        }
    }
    
    /**
     * Parse_Icon_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Icon_Data(/*** string ***/ $line)
    {
        $arr = $this->_parseByRegex("/\[(\d+)\].+\('(.*)'(,(\d+),(\d+))?\)$/",$line);
        
        $icon = $this->_mHandlers[SD_OBJECT_TYPE_ICON]->getByUnique($this->_mActiveId,$arr[1]);
        if($icon->get('icon') != $arr[2])
        {
            $icon->setVars(
                array(
                    'eno'         => $this->_mActiveId,
                    'slot'        => $arr[1],
                    'icon'        => $arr[2],
                    'icon_height' => isset($arr[5]) ? $arr[5] : 0,
                    'icon_width'  => isset($arr[4]) ? $arr[4] : 0,
                    'update_time' => SD_UPDATE_TIME
                )
            );
            $this->_insert(SD_OBJECT_TYPE_ICON,$icon);
        }
    }
}

?>
