<?php
/**
 * @file
 * @package sd
 * @version $Id$
**/

if(!defined('XOOPS_ROOT_PATH'))
{
    exit();
}


define('SD_ENUM_TYPE_ITEM_NORMAL',0);
define('SD_ENUM_TYPE_ITEM_DOUBLE',1);
define('SD_ENUM_TYPE_ITEM_SINGLE',2);


/**
 * @class   Sd_ResultUpdater
**/
class Sd_ResultUpdater extends Sd_AbstractJsUpdater
{
    /**
     * @brief   Enum[]
    **/
    protected $_mHandlerList = array(
        SD_OBJECT_TYPE_CHAR,
        SD_OBJECT_TYPE_DROP,
        SD_OBJECT_TYPE_DROP_SEED,
        SD_OBJECT_TYPE_ENEMY,
        SD_OBJECT_TYPE_ENEMY_SKILL,
        SD_OBJECT_TYPE_ITEM,
        SD_OBJECT_TYPE_ITEM_SLOT,
        SD_OBJECT_TYPE_MAP,
        SD_OBJECT_TYPE_MAP_ENEMY_LINK,
        SD_OBJECT_TYPE_MAP_SPOT,
        SD_OBJECT_TYPE_PARTY,
        SD_OBJECT_TYPE_PNO_DROP_LINK,
        SD_OBJECT_TYPE_PNO_ENEMY_LINK,
        SD_OBJECT_TYPE_PNO_ITEM_LINK,
        SD_OBJECT_TYPE_ENUM
    );
    
    /**
     * @brief   bool
    **/
    protected $_mEndFlag = false;
    
    /**
     * @brief   Enum{}
    **/
    protected $_mMap = array();
    
    /**
     * @brief   Sd_Party
    **/
    protected $_mParty = null;
    
    /**
     * @brefi   int{}
    **/
    protected $_mEno = array();
    
    /**
     * @brief   int{}
    **/
    protected $_mEnemy = array();
    
    /**
     * @brief   Sd_Item_slotObject{}
    **/
    protected $_mDrop = array();
    
    /**
     * _save
     * 
     * @param   void
     * 
     * @return  void
    **/
    protected function _save()
    {
        if(count($this->_mEno) > 0)
        {
            $this->_insert(SD_OBJECT_TYPE_PARTY,$this->_mParty);
        }
        else
        {
            $this->_delete(SD_OBJECT_TYPE_PARTY,$this->_mParty);
        }
    }
    
    /**
     * Parse_Name_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Name_Data(/*** string ***/ $line)
    {
        $handler =& $this->_mHandlers[SD_OBJECT_TYPE_PARTY];
        if($this->_mParty instanceof $handler->mClass)
        {
            return;
        }
        
        $arr = $this->_parseByRegex('/\[(\d+)\] = "(.+)";$/',$line);
        
        $this->_mParty = $handler->get($this->_mActiveId);
        if(!$this->_mParty instanceof $handler->mClass)
        {
            $this->_mParty = $handler->create();
            $this->_mParty->set('pno',$this->_mActiveId);
        }
        
        $this->_mParty->refleshResult($arr[2]);
    }
    
    /**
     * Parse_Party_Join
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Party_Join(/*** string ***/ $line)
    {
        $arr = $this->_parseByExplode($line,5,11,-1);
        
        if($arr[4] == 0)
        {
            $root =& XCube_Root::getSingleton();
            $backPoint = $root->mContext->getAttribute('sd.party.backPoint');
            if(isset($backPoint[$arr[1]]))
            {
                $this->_mParty->setVars(array('x' => $backPoint[$arr[1]]['x'],'y' => $backPoint[$arr[1]]['y']));
            }
        }
    }
    
    /**
     * Parse_Map_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Map_Data(/*** string ***/ $line)
    {
        $arr = $this->_parseByRegex("/\[(\d+)\] = .+\('.+','(.+)'\)$/",$line);
        $handler =& $this->_mHandlers[SD_OBJECT_TYPE_MAP_SPOT];
        
        $spot = $handler->getByName($arr[2]);
        if(!$spot instanceof $handler->mClass)
        {
            $spot = $handler->create();
            $spot->setVars(array('spot_name' => $arr[2],'update_time' => SD_UPDATE_TIME));
            $this->_insert(SD_OBJECT_TYPE_MAP_SPOT,$spot);
        }
        
        $this->_mMap[$arr[1]] = $spot->get('spot_id');
    }
    
    /**
     * Parse_Show_Map
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Show_Map(/*** string ***/ $line)
    {
        $handler =& $this->_mHandlers[SD_OBJECT_TYPE_MAP];
        $baseX = $this->_mParty->get('x') - 4;
        $baseY = $this->_mParty->get('y') - 4;
        $arr = array();
        
        if(SD_UPDATE_TIME <= 80123)
        {
            $arr = $this->_parseByExplode($line,81,9,-1);
        }
        else
        {
            $arr = $this->_parseByExplode($line,82,9,-1);
            if(array_shift($arr) == 1)
            {
//                $this->_mEndFlag = true;
            }
        }
        
        for($i = 0;$i < count($arr);$i++)
        {
            $x = $i % 9 + $baseX;
            $y = intval($i / 9) + $baseY;
            $spot = $this->_mMap[$arr[$i]];
            
            if($spot != 1)
            {
                $point = $handler->get($x,$y);
                if($spot != $point->get('spot_id'))
                {
                    $point->setVars(array('spot_id' => $spot,'update_time' => SD_UPDATE_TIME));
                    $this->_insert(SD_OBJECT_TYPE_MAP,$point);
               }
            }
        }
    }
    
    /**
     * Parse_MapMove
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_MapMove(/*** string ***/ $line)
    {
        $arr = $this->_parseByExplode($line,2,8,-1);
        $direction = trim($arr[0],"'");
        
        if($direction != _MD_SD_LABEL_DIRECTION0)
        {
            $this->_mParty->moveByDirection($direction);
        }
        else
        {
            $this->Parse_MapBack($line);
        }
    }
    
    /**
     * Parse_Map_Move
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Map_Move(/*** string ***/ $line)
    {
        $arr = $this->_parseByExplode($line,3,9,-1);
        $value = trim($arr[1],"'");
        
        switch($arr[0])
        {
            case -1:
                break;
            case 0:
                if($value != _MD_SD_LABEL_DIRECTION0)
                {
                    $this->_mParty->moveByDirection($value);
                }
                else
                {
                    $this->Parse_MapBack($line);
                }
                break;
            case 1:
                break;
            case 2:
                if($value != 0 && isset($this->_mMap[$arr[2]]))
                {
                    $cri = new Criteria('spot_id',$this->_mMap[$arr[2]]);
                    $map = $this->_mHandlers[SD_OBJECT_TYPE_MAP]->getObjects($cri);
                    
                    if(!is_array($map) || count($map) != 1)
                    {
                        $this->_displayError(array('spot' => $spot,'map' => $map));
                    }
                    
                    $this->_mParty->setPoint($map[0]->get('x'),$map[0]->get('y'));
                }
                break;
        }
    }
    
    /**
     * Parse_MapBack
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_MapBack(/*** string ***/ $line)
    {
        $root =& XCube_Root::getSingleton();
        $backPoint = $root->mContext->getAttribute('sd.party.backPoint');
        $this->_mParty->setVars(array('x' => $backPoint[$this->_mActiveId]['x'],'y' => $backPoint[$this->_mActiveId]['y']));
    }
    
    /**
     * Parse_Show_Status
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    public function Parse_Show_Status(/*** string ***/ $line)
    {
        if($this->_mEndFlag)
        {
            return;
        }
        
        $arr = $this->_parseByRegex('/\((\d+),/',$line);
        $this->_mEno[] = $arr[1];
    }
    
    /**
     * Parse_Struct_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Struct_Data(/*** string ***/ $line)
    {
        $arr = $this->_parseByRegex("/\[(\d)\] = .+\((\d+),'(.+)','.+',\d+,'.*','.*','(.+)'\)$/",$line);
        
        if($arr[2] == 0)
        {
            $handler =& $this->_mHandlers[SD_OBJECT_TYPE_CHAR];
            $char = $handler->get($this->_mEno[$arr[1]]);
            if($char instanceof $handler->mClass)
            {
                $char->set('battle_name',$arr[3]);
                $this->_insert(SD_OBJECT_TYPE_CHAR,$char);
            }
        }
        else
        {
            $enemyHandler =& $this->_mHandlers[SD_OBJECT_TYPE_ENEMY];
            $mapHandler =& $this->_mHandlers[SD_OBJECT_TYPE_MAP_ENEMY_LINK];
            $enemy = $enemyHandler->getByName($arr[3]);
            
            if(!$enemy instanceof $enemyHandler->mClass)
            {
                $enemy = $enemyHandler->create();
                $enemy->setVars(
                    array(
                        'enemy_name'  => $arr[3],
                        'attack_kind' => $this->_mHandlers[SD_OBJECT_TYPE_ENUM]->convertToData(SD_ENUM_TYPE_TYPE,$arr[4]),
                        'regist_time' => SD_UPDATE_TIME
                    )
                );
                $this->_insert(SD_OBJECT_TYPE_ENEMY,$enemy);
            }
            
            $link = $this->_mHandlers[SD_OBJECT_TYPE_PNO_ENEMY_LINK]->create();
            $link->setVars(array('pno' => $this->_mActiveId,'enemy_id' => $enemy->get('enemy_id')));
            $this->_insert(SD_OBJECT_TYPE_PNO_ENEMY_LINK,$link);
            
            $map = $mapHandler->get($this->_mParty->get('x'),$this->_mParty->get('y'),$enemy->get('enemy_id'));
            if(!$map instanceof $mapHandler->mClass)
            {
                $map = $mapHandler->create();
                $map->setVars(array('x' => $this->_mParty->get('x'),'y' => $this->_mParty->get('y'),'enemy_id' => $enemy->get('enemy_id')));
                $this->_insert(SD_OBJECT_TYPE_MAP_ENEMY_LINK,$map);
            }
            
            $this->_mEnemy[$arr[1]] = $enemy->get('enemy_id');
        }
    }
    
    /**
     * Parse_Show_Turn
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Show_Turn(/*** string ***/ $line)
    {
        $arr = $this->_parseByRegex('/\((\d+),/',$line);
        $this->_mParty->set('turn',$arr[1]);
    }
    
    /**
     * Parse_AttackReady
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_AttackReady(/*** string ***/ $line)
    {
        $arr = $this->_parseByRegex("/\((\d+),\-?\d+,\d+,(?:\-?\d+|'[^']+'),(0|'[^']+'),.+\)$/",$line);
        $name = strpos($arr[2],"'") === 0 ? trim($arr[2],"'") : null;
        
        if(isset($this->_mEnemy[$arr[1]]) && $name != null)
        {
            $handler =& $this->_mHandlers[SD_OBJECT_TYPE_ENEMY_SKILL];
            $id = $this->_mEnemy[$arr[1]];
            $skill = $handler->getByUnique($id,$name);
            
            if(!$skill instanceof $handler->mClass)
            {
                $skill = $handler->create();
                $skill->setVars(array('enemy_id' => $id,'skill_name' => $name));
                $this->_insert(SD_OBJECT_TYPE_ENEMY_SKILL,$skill);
            }
        }
    }
    
    /**
     * Parse_BattleEnd
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_BattleEnd(/*** string ***/ $line)
    {
        $res = substr($line,10,1) + 2;
        $this->_mParty->set('result',$res);
        $this->_mEndFlag = true;
    }
    
    /**
     * Parse_BornusGet
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_BornusGet(/*** string ***/ $line)
    {
        $arr = array();
        if(preg_match_all('/(\w+)\(([^\)]+)\)/',substr($line,12,-1),$arr))
        {
            foreach(array_keys($arr[0]) as $key)
            {
                $func = 'subParse_' . $arr[1][$key];
                if(method_exists($this,$func))
                {
                    $this->$func($arr[2][$key]);
                }
            }
        }
    }
    
    /**
     * subParse_ExpPointGet
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function subParse_ExpPointGet(/*** string ***/ $line)
    {
        $arr = explode(',',$line);
        $this->_mParty->set('get_exp',$arr[0]);
    }
    
    /**
     * subParse_SoulJunkGet
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function subParse_SoulJunkGet(/*** string ***/ $line)
    {
        $this->_mParty->set('get_money',$line);
    }
    
    /**
     * subParse_ItemGet
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function subParse_ItemGet(/*** string ***/ $line)
    {
        // $arr = explode(',',substr($line,8,-1));
        
        // for($i = 1;$i < count($arr);$i += 2)
        // {
            // $this->_mItems[] = trim($arr[$i],"'");
        // }
    }
    
    /**
     * subParse_SDropGet
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function subParse_SDropGet(/*** string ***/ $line)
    {
        // $arr = explode(',',substr($line,9,1));
        
        // for($i = 1;$i < count($arr);$i += 2)
        // {
            // $this->_mDrops[] = trim($arr[$i],"'");
        // }
    }
    
    /**
     * Parse_BackHome
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_BackHome(/*** string ***/ $line)
    {
        $spot = $this->_mHandlers[SD_OBJECT_TYPE_MAP_SPOT]->getByName(substr($line,10,-2));
        $cri = new Criteria('spot_id',$spot->get('spot_id'));
        $map = $this->_mHandlers[SD_OBJECT_TYPE_MAP]->getObjects($cri);
        
        if(!is_array($map) || count($map) != 1)
        {
            $this->_displayError(array('spot' => $spot,'map' => $map));
        }
        
        $this->_mParty->setPoint($map[0]->get('x'),$map[0]->get('y'));
    }
    
    /**
     * Parse_Item_Data
     * 
     * @param   string  $line
     * 
     * @return  void
    **/
    protected function Parse_Item_Data(/*** string ***/ $line)
    {
        if(!$this->_mEndFlag)
        {
            return;
        }
        
        $itemHandler =& $this->_mHandlers[SD_OBJECT_TYPE_ITEM];
        $dropHandler =& $this->_mHandlers[SD_OBJECT_TYPE_DROP];
        $linkHandler =& $this->_mHandlers[SD_OBJECT_TYPE_PNO_ITEM_LINK];
        $enumHandler =& $this->_mHandlers[SD_OBJECT_TYPE_ENUM];
        
        $arr = $this->_parseByRegex("/\(\d+,(\d+),\d+,'\w+','([^']+)','([^']+)',(\d+),(\d+),(.+)\)$/",$line);
        $detail = explode(',',$arr[6]);
        $slot = array();
        $item = $itemHandler->getByName($arr[2]);
        
        if(!$item instanceof $itemHandler->mClass)
        {
            $item = $itemHandler->create();
            $item->setVars(
                array(
                    'item_name' => $arr[2],
                    'kind'      => $enumHandler->convertToData(SD_ENUM_TYPE_TYPE,$arr[3]),
                    'lv'        => $arr[4],
                    'element'   => $arr[5],
                    'regist_time' => SD_UPDATE_TIME
                )
            );
            
            switch($arr[1])
            {
                case SD_ENUM_TYPE_ITEM_NORMAL:
                    $item->setVars(
                        array(
                            'at'  => $detail[0],
                            'df'  => $detail[1],
                            'mat' => $detail[2],
                            'mdf' => $detail[3],
                            'range' => trim($detail[4],"'") * 10
                        )
                    );
                    
                    for($i = 1;$i <= count($detail) - 5;$i++)
                    {
                        if(($num = $detail[$i + 4]) != -3)
                        {
                            $slot[$i] = $num;
                        }
                    }
                    
                    break;
                case SD_ENUM_TYPE_ITEM_DOUBLE:
                    break;
                case SD_ENUM_TYPE_ITEM_SINGLE:
                    break;
                default:
                    break;
            }
            
            $this->_insert(SD_OBJECT_TYPE_ITEM,$item);
            
            foreach($slot as $k => $v)
            {
                $s = $this->_mHandlers[SD_OBJECT_TYPE_ITEM_SLOT]->create();
                $s->setVars(array('item_id' => $item->get('item_id'),'slot' => $k));
                if($v < 0)
                {
                    $s->set('drop_id',abs($v));
                    $this->_insert(SD_OBJECT_TYPE_ITEM_SLOT,$s);
                }
                else
                {
                    $this->_mDrop[$v] = $s;
                }
            }
        }
        
        $link = $linkHandler->create();
        $link->setVars(array('pno' => $this->_mActiveId,'item_id' => $item->get('item_id')));
        $this->_insert(SD_OBJECT_TYPE_PNO_ITEM_LINK,$link);
    }
    
    /**
     * Parse_SDrop_Data
     * 
     * @param   string $line
     * 
     * @return  void
    **/
    protected function Parse_SDrop_Data(/*** string ***/ $line)
    {
        if(!$this->_mEndFlag)
        {
            return;
        }
        
        $dropHandler =& $this->_mHandlers[SD_OBJECT_TYPE_DROP];
        $seedHandler =& $this->_mHandlers[SD_OBJECT_TYPE_DROP_SEED];
        
        $arr = $this->_parseByRegex("/\[(\d+)\] = .+\((?:\d+,)?'(.+)','(.+)',(\d+),(?:(\d+),)?'(.*)'\)$/",$line);
        $drop = $dropHandler->getByUnique($arr[2],$arr[4],$arr[6]);
        $seed = $seedHandler->getByName($arr[3]);
        $num = $arr[1];
        $ele = $arr[5] != null ? $arr[5] : -2;
        
        if(!$drop instanceof $dropHandler->mClass)
        {
            if(!$seed instanceof $seedHandler->mClass)
            {
                $seed = $seedHandler->create();
                $seed->set('drop_seed_name',$arr[3]);
                $this->_insert(SD_OBJECT_TYPE_DROP_SEED,$seed);
            }
            
            $drop = $dropHandler->create();
            $drop->setVars(
                array(
                    'drop_name'   => $arr[2],
                    'seed'        => $seed->get('drop_seed_id'),
                    'lv'          => $arr[4],
                    'element'     => $arr[5],
                    'gift'        => $arr[6],
                    'regist_time' => SD_UPDATE_TIME
                )
            );
            
            $this->_insert(SD_OBJECT_TYPE_DROP,$drop);
        }
        
        if(isset($this->_mDrop[$num]))
        {
            $this->_mDrop[$num]->set('drop_id',$drop->get('drop_id'));
            $this->_insert(SD_OBJECT_TYPE_ITEM_SLOT,$this->_mDrop[$num]);
        }
        else
        {
            $link = $this->_mHandlers[SD_OBJECT_TYPE_PNO_DROP_LINK]->create();
            $link->setVars(array('pno' => $this->_mActiveId,'drop_id' => $drop->get('drop_id')));
            $this->_insert(SD_OBJECT_TYPE_PNO_DROP_LINK,$link);
        }
    }
}

?>
